import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as stats

matplotlib.rcParams["savefig.dpi"] = 300
matplotlib.rcParams["figure.dpi"] = 200


def plot_normal_distribution(data: list) -> None:
    data.sort()
    mean_ = np.mean(data)
    std_ = np.std(data)
    pdf = stats.norm.pdf(data, mean_, std_)
    plt.plot(data, pdf)
    plt.show()


def plot_wordcloud(wordcloud) -> None:
    plt.figure(dpi=360)
    plt.imshow(wordcloud, interpolation="bilinear")
    plt.axis("off")
    plt.show()


def plot_bars_from_dict(
    dict_: dict,
    labelsize=None,
    figure: tuple[int, int, int] = None,
    title="Chart",
    style="seaborn",
) -> None:
    dict_ = {k: v for k, v in sorted(dict_.items(), key=lambda item: item[1])}
    print(f"Plotting this data: {dict_}")
    plt.style.use(style)
    x = list(dict_.keys())
    y = list(dict_.values())

    if figure:
        plt.figure(figsize=(figure[0], figure[1]), dpi=figure[2])
    if labelsize:
        plt.tick_params(which="minor", labelsize=labelsize)
        plt.tick_params(which="major", labelsize=labelsize)
    plt.barh(x, y, color="green")
    plt.title(title)
    plt.show()
    plt.savefig("image.png", dpi=200)



import datetime
import json
import re
from collections import Counter
from collections.abc import Iterable
from typing import Generator

import nltk
from nltk.corpus import stopwords

from wordcloud import WordCloud
from loguru import logger
from pyrogram.types.user_and_chats.user import User
from dotenv import load_dotenv
from stop_words import get_stop_words

from telenal.teleplotter import plot_wordcloud, plot_bars_from_dict

load_dotenv()


class Chats:
    # an object that describes a json export and helps do actions with it
    ...


class TeleUser(User):
    ...
    # def __init__(self, user: User):
    # super().__init__()
    # pass


def json_generator(json_path: str, target: str | list[str]) -> Generator:
    try:
        with open(json_path, encoding="utf-8", mode="r") as json_file:
            json_data = json.load(json_file)
            if isinstance(json_data[target], Iterable):
                item: dict
                for item in json_data[target]:
                    yield item
            else:
                raise Exception(
                    f"Incorrect Target, {target} should route to an Iterable"
                )
    except ValueError as error:
        logger.error(error)


def who_breaks_silence_json(
    json_path: str, pause_in_hours=1, out="username", plot=True
) -> Counter:
    """A FUNction to detect, who breaks silence in a given chat after a {pause_in_hours} hour pause"""
    out = ("id", "from")[out == "username"]
    json_gen: Generator = json_generator(json_path, target="messages")
    json_gen.__next__()  # <-- removes first "group item" which is not a correct message, to be improved
    before_message = json_gen.__next__()
    assert before_message["type"] == "message", Exception(
        "Wrong Data, Got non-message object"
    )
    breaker_counter = Counter()
    for after_message in json_gen:
        after_date = datetime.datetime.strptime(
            after_message["date"], "%Y-%m-%dT%H:%M:%S"
        )
        before_date = datetime.datetime.strptime(
            before_message["date"], "%Y-%m-%dT%H:%M:%S"
        )
        timedelta = after_date - before_date

        if (timedelta.seconds / 3600) > pause_in_hours:
            print(
                f"({after_date}): {after_message['text'] or 'non-text'} is the first message",
                end="\r",
            )
            if after_message.get("from", False):
                breaker_counter.update([after_message[out]])
        before_message = after_message
    print(f"\n\nSilence breakers: {dict(breaker_counter)}")
    if plot:
        if len(dict(breaker_counter)) >= 100:
            labelsize = 3
        else:
            labelsize = 7
        plot_bars_from_dict(dict(breaker_counter), labelsize=labelsize)
    return breaker_counter


def wordcloud_json(
    json_path: str,
    language="ukrainian",
    plot=True,
    custom_stopwords: None | set = None,
    max_words=400,
) -> WordCloud:
    json_gen: Generator = json_generator(json_path, target="messages")
    json_gen.__next__()  # <-- removes first "group item" which is not a correct message, to be improved
    if language == "ukrainian":
        # fmt: off
        stop_words = set(get_stop_words("russian")) | {'а', 'аби', 'абиде', 'абиким', 'абикого', 'абиколи', 'абикому', 'абикуди', 'абихто', 'абичий', 'абичийого', 'абичийому', 'абичим', 'абичию', 'абичия', 'абичиє', 'абичиєму', 'абичиєю', 'абичиєї', 'абичиї', 'абичиїй', 'абичиїм', 'абичиїми', 'абичиїх', 'абичого', 'абичому', 'абищо', 'абияка', 'абияке', 'абиякий', 'абияким', 'абиякими', 'абияких', 'абиякого', 'абиякому', 'абиякою', 'абиякої', 'абияку', 'абиякі', 'абиякій', 'абиякім', 'або', 'абощо', 'авжеж', 'авось', 'ага', 'ад', 'адже', 'аж', 'ажень', 'аз', 'ай', 'але', 'ало', 'амінь', 'ант', 'ану', 'ані', 'аніде', 'аніж', 'анізащо', 'аніким', 'анікого', 'анікогісінько', 'аніколи', 'анікому', 'аніскільки', 'аніхто', 'анічим', 'анічого', 'анічогісінько', 'анічому', 'аніщо', 'аніяка', 'аніяке', 'аніякий', 'аніяким', 'аніякими', 'аніяких', 'аніякого', 'аніякому', 'аніякою', 'аніякої', 'аніяку', 'аніякі', 'аніякій', 'аніякім', 'аніякісенька', 'аніякісеньке', 'аніякісенький', 'аніякісеньким', 'аніякісенькими', 'аніякісеньких', 'аніякісенького', 'аніякісенькому', 'аніякісенькою', 'аніякісенької', 'аніякісеньку', 'аніякісенькі', 'аніякісенькій', 'аніякісенькім', 'аніякісінька', 'аніякісіньке', 'аніякісінький', 'аніякісіньким', 'аніякісінькими', 'аніякісіньких', 'аніякісінького', 'аніякісінькому', 'аніякісінькою', 'аніякісінької', 'аніякісіньку', 'аніякісінькі', 'аніякісінькій', 'аніякісінькім', 'ат', 'ато', 'атож', 'ау', 'ах', 'ач', 'ачей', 'аякже', 'б', 'ба', 'багато', 'багатьма', 'багатьом', 'багатьох', 'баз', 'бай', 'бат', 'бах', 'бац', 'баш', 'бе', 'беж', 'без', 'безперервно', 'бел', 'бер', 'би', 'бир', 'бич', 'близько', 'близько від', 'бо', 'бов', 'бод', 'бодай', 'боз', 'бош', 'був', 'буває', 'буде', 'будем', 'будемо', 'будете', 'будеш', 'буду', 'будуть', 'будь', 'будь ласка', 'будьмо', 'будьте', 'була', 'були', 'було', 'бути', 'бух', 'буц', 'буцім', 'буцімто', 'бі', 'біб', 'більш', 'більше', 'біля', 'в', 'в бік', 'в залежності від', 'в міру', 'в напрямі до', 'в порівнянні з', 'в процесі', 'в результаті', 'в ролі', 'в силу', 'в сторону', 'в супроводі', 'в ході', "в ім'я", 'в інтересах', 'вад', 'важлива', 'важливе', 'важливий', 'важливі', 'вак', 'вам', 'вами', 'ван', 'вас', 'ват', 'ваш', 'ваша', 'ваше', 'вашим', 'вашими', 'ваших', 'вашого', 'вашому', 'вашою', 'вашої', 'вашу', 'ваші', 'вашій', 'вашім', 'ввесь', 'вві', 'вгору', 'вдалині', 'вед', 'верх', 'весь', 'вех', 'вже', 'вздовж', 'ви', 'виз', 'вис', 'височині', 'вище ', 'вйо', 'власне', 'властиво', 'вміти', 'внаслідок', 'вниз', 'внизу', 'во', 'вон', 'вона', 'вони', 'воно', 'восьмий', 'вперед', 'вподовж', 'впоперек', 'впритиск', 'впритул', 'впродовж', 'впрост', 'все', 'всередині', 'всею', 'вслід', 'всупереч', 'всього', 'всьому', 'всю', 'всюди', 'вся', 'всяк', 'всяка', 'всяке', 'всякий', 'всяким', 'всякими', 'всяких', 'всякого', 'всякому', 'всякою', 'всякої', 'всяку', 'всякі', 'всякій', 'всякім', 'всі', 'всій', 'всіляка', 'всіляке', 'всілякий', 'всіляким', 'всілякими', 'всіляких', 'всілякого', 'всілякому', 'всілякою', 'всілякої', 'всіляку', 'всілякі', 'всілякій', 'всілякім', 'всім', 'всіма', 'всіх', 'всією', 'всієї', 'втім', 'ві', 'віг', 'від', 'від імені', 'віддалік від', 'відколи', 'відносно', 'відповідно', 'відповідно до', 'відсотків', 'відтепер', 'відтоді', 'він', 'вісім', 'вісімнадцятий', 'вісімнадцять', 'віт', 'віф', 'віх', 'віц', 'віщо', 'віщось', 'г', 'га', 'гав', 'гаразд', 'ге', 'гез', 'гем', 'геп', 'гет', 'геть', 'гех', 'ги', 'гик', 'гир', 'гич', 'гм', 'го', 'говорив', 'гог', 'гоп', 'гоц', 'гу', 'гуп', 'д', 'да', 'давай', 'давати', 'давно', 'далеко', 'далеко від', 'далі', 'даром', 'два', 'двадцятий', 'двадцять', 'дванадцятий', 'дванадцять', 'двох', 'дві', 'де', "дев'ятий", "дев'ятнадцятий", "дев'ятнадцять", "дев'ять", 'дедалі', 'деким', 'декого', 'деколи', 'декому', 'декотра', 'декотре', 'декотрий', 'декотрим', 'декотрими', 'декотрих', 'декотрого', 'декотрому', 'декотрою', 'декотрої', 'декотру', 'декотрі', 'декотрій', 'декотрім', 'декілька', 'декільком', 'декількома', 'декількох', 'декім', 'десь', 'десятий', 'десять', 'дехто', 'дечий', 'дечийого', 'дечийому', 'дечим', 'дечию', 'дечия', 'дечиє', 'дечиєму', 'дечиєю', 'дечиєї', 'дечиї', 'дечиїй', 'дечиїм', 'дечиїми', 'дечиїх', 'дечого', 'дечому', 'дечім', 'дещо', 'деяка', 'деяке', 'деякий', 'деяким', 'деякими', 'деяких', 'деякого', 'деякому', 'деякою', 'деякої', 'деяку', 'деякі', 'деякій', 'деякім', 'деінде', 'для', 'до', 'добре', 'довго', 'довкола', 'довкіл', 'дог', 'доки', 'допоки', 'допіру', 'досить', 'досі', 'дотепер', 'доти', 'другий', 'друго', 'дуже', 'дякую', 'дійсно', 'діл', 'е', 'еге', 'еж', 'ей', 'ерг', 'ест', 'ет', 'ех', 'еч', 'ж', 'же', 'жоден', 'жодна', 'жодне', 'жодний', 'жодним', 'жодними', 'жодних', 'жодного', 'жодному', 'жодною', 'жодної', 'жодну', 'жодні', 'жодній', 'жоднім', 'жоднісінька', 'жоднісіньке', 'жоднісінький', 'жоднісіньким', 'жоднісінькими', 'жоднісіньких', 'жоднісінького', 'жоднісінькому', 'жоднісінькою', 'жоднісінької', 'жоднісіньку', 'жоднісінькі', 'жоднісінькій', 'жоднісінькім', 'жуз', 'з', 'з метою', 'з нагоди', 'з приводу', 'з розрахунку на', 'з-за', 'з-над', 'з-перед', 'з-поза', 'з-поміж', 'з-понад', 'з-поперед', 'з-посеред', 'з-проміж', 'з-під', 'з-серед', 'за', 'за винятком', 'за допомогою', 'за посередництвом', 'за рахунок', 'завгодно', 'завдяки', 'завжди', 'завше', 'задля', 'зазвичай', 'зайнята', 'зайнятий', 'зайнято', 'зайняті', 'залежно', 'залежно від', 'замість', 'занадто', 'заради', 'зараз', 'зас', 'зате', 'збоку', 'збоку від', 'зважаючи на', 'зверх ', 'зверху', 'звичайно', 'звиш', 'звідки', 'звідкилясь', 'звідкись', 'звідкіль', 'звідкіля', 'звідкілясь', 'звідси', 'звідсіль', 'звідсіля', 'звідти', 'звідтіль', 'звідтіля', 'звідусюди', 'звідусіль', 'звідціля', 'згідно з', 'здається', 'здовж', 'зем', 'зет', 'ззаду', 'зиз', 'зик', 'значить', 'знову', 'зо', 'зовсім', 'зсередини', 'зух', 'зі', 'зіс', 'и', 'ич', 'й', 'ймовірно', 'йно', 'йо', 'його', 'йой', 'йол', 'йому', 'йор', 'йот', 'йох', 'к', 'каже', 'каз', 'кар', 'каф', 'ках', 'ке', 'кед', 'кет', 'кеш', 'кив', 'кий', 'кил', 'ким', 'кимось', 'кимсь', 'ких', 'киш', 'коб', 'коби', 'кого', 'когось', 'кожен', 'кожна', 'кожне', 'кожний', 'кожним', 'кожними', 'кожних', 'кожного', 'кожному', 'кожною', 'кожної', 'кожну', 'кожні', 'кожній', 'кожнім', 'кожнісінька', 'кожнісіньке', 'кожнісінький', 'кожнісіньким', 'кожнісінькими', 'кожнісіньких', 'кожнісінького', 'кожнісінькому', 'кожнісінькою', 'кожнісінької', 'кожнісіньку', 'кожнісінькі', 'кожнісінькій', 'кожнісінькім', 'коли', 'колись', 'коло', 'кому', 'комусь', 'котра', 'котрась', 'котре', 'котресь', 'котрий', 'котрийсь', 'котрим', 'котрими', 'котримись', 'котримось', 'котримсь', 'котрих', 'котрихось', 'котрихсь', 'котрого', 'котрогось', 'котрому', 'котромусь', 'котрою', 'котроюсь', 'котрої', 'котроїсь', 'котру', 'котрусь', 'котрі', 'котрій', 'котрійсь', 'котрім', 'котрімсь', 'котрісь', 'коц', 'коч', 'коштом', 'край', 'краще', 'кру', 'круг', 'кругом', 'крю', 'кря', 'крізь', 'крім', 'куди', 'кудись', 'кудою', 'кілька', 'кільком', 'кількома', 'кількох', 'кім', 'кімось', 'кімсь', 'кінець', 'л', 'лаж', 'лап', 'лас', 'лат', 'ле', 'ледве', 'ледь', 'лет', 'лиш', 'лише', 'лишень', 'лум', 'луп', 'лут', 'льє', 'люди', 'людина', 'ля', 'лі', 'ліворуч від', 'лік', 'лім', 'м', 'мабуть', 'майже', 'мало', 'мати', 'мац', 'ме', 'меж', 'мене', 'менше', 'мені', 'мерсі', 'мет', 'мжа', 'ми', 'мимо ', 'миру', 'мит', 'мною', 'мо', 'мов', 'мовби', 'мовбито', 'могла', 'могли', 'могло', 'мого', 'могти', 'мож', 'може', 'можем', 'можемо', 'можете', 'можеш', 'можна', 'можу', 'можуть', 'можіть', 'мой', 'мол', 'мою', 'моя', 'моє', 'моєму', 'моєю', 'моєї', 'мої', 'моїй', 'моїм', 'моїми', 'моїх', 'му', 'мі', 'міг', 'між', 'мій', 'мільйонів', 'н', 'на', 'на адресу', 'на базі', 'на благо', 'на випадок', 'на відміну від', 'на засадах', 'на знак', 'на зразок', 'на користь', 'на кшталт', 'на межі', 'на основі', 'на противагу', 'на підставі', 'на честь', 'на чолі', 'на ґрунті', 'навколо', 'навкруг', 'навкруги ', 'навкіл', 'навпаки', 'навперейми', 'навпроти', 'навіть', 'навіщо', 'навіщось', 'нагорі', 'над', 'надо', 'надовкола', 'надокола', 'наді', 'назавжди', 'назад', 'назустріч', 'най', 'найбільш', 'нам', 'нами', 'наоколо ', 'наокруг ', 'наокруги ', 'наокіл', 'наперед', 'напередодні', 'напереді', 'наперекір', 'напереріз', 'наприкінці', 'напроти', 'нарешті', 'нарівні з', 'нас', 'насеред', 'насподі', 'наспід', 'настрічу', 'насупроти', 'насупротив ', 'нате', 'наче', 'начеб', 'начебто', 'наш', 'наша', 'наше', 'нашим', 'нашими', 'наших', 'нашого', 'нашому', 'нашою', 'нашої', 'нашу', 'наші', 'нашій', 'нашім', 'не', 'не до', 'не можна', 'неабичим', 'неабичого', 'неабичому', 'неабищо', 'небагато', 'небагатьма', 'небагатьом', 'небагатьох', 'небудь', 'невважаючи', 'невже', 'недалеко', 'недалеко від', 'неж', 'незалежно від', 'незважаючи', 'незважаючи на', 'ней', 'немає', 'немов', 'немовби', 'немовбито', 'неначе', 'неначебто', 'неподалеку', 'неподалеку від', 'неподалечку', 'неподалечку від', 'неподалік', 'неподалік від', 'нерідко', 'нех', 'нехай', 'нещодавно', 'нею', 'неї', 'нижче', 'низько', 'ник', 'ним', 'ними', 'них', 'нич', 'но', 'ну', 'нуг', 'нуд', 'нум', 'нумо', 'нумте', 'ньо', 'нього', 'ньому', 'ню', 'нюх', 'ня', 'няв', 'ні', 'ніби', 'ніби-то', 'нібито', 'ніде', 'ніж', 'нізащо', 'нізвідки', 'нізвідкіля', 'ній', 'ніким', 'нікого', 'нікогісінько', 'ніколи', 'нікому', 'нікотра', 'нікотре', 'нікотрий', 'нікотрим', 'нікотрими', 'нікотрих', 'нікотрого', 'нікотрому', 'нікотрою', 'нікотрої', 'нікотру', 'нікотрі', 'нікотрій', 'нікотрім', 'нікуди', 'нім', 'нінащо', 'ніскільки', 'ніт', 'ніхто', 'нічий', 'нічийна', 'нічийне', 'нічийний', 'нічийним', 'нічийними', 'нічийних', 'нічийного', 'нічийному', 'нічийною', 'нічийної', 'нічийну', 'нічийні', 'нічийній', 'нічийнім', 'нічийого', 'нічийому', 'нічим', 'нічию', 'нічия', 'нічиє', 'нічиєму', 'нічиєю', 'нічиєї', 'нічиї', 'нічиїй', 'нічиїм', 'нічиїми', 'нічиїх', 'нічого', 'нічому', 'ніщо', 'ніяк', 'ніяка', 'ніяке', 'ніякий', 'ніяким', 'ніякими', 'ніяких', 'ніякого', 'ніякому', 'ніякою', 'ніякої', 'ніяку', 'ніякі', 'ніякій', 'ніякім', 'ніякісінька', 'ніякісіньке', 'ніякісінький', 'ніякісіньким', 'ніякісінькими', 'ніякісіньких', 'ніякісінького', 'ніякісінькому', 'ніякісінькою', 'ніякісінької', 'ніякісіньку', 'ніякісінькі', 'ніякісінькій', 'ніякісінькім', 'о', 'об', 'обабіч', 'обаполи', 'обидва', 'обр', 'обік', 'обіруч', 'обіч', 'ов', 'од', 'один', 'одинадцятий', 'одинадцять', 'одна', 'однак', 'одначе', 'одне', 'одним', 'одними', 'одних', 'одно', 'одного', 'одного разу', 'одному', 'одною', 'одної', 'одну', 'одні', 'одній', 'однім', 'однією', 'однієї', 'ож', 'ой', 'окрай', 'окроме', 'округ', 'округи', 'окрім', 'окіл', 'ом', 'он', 'онде', 'онно', 'оно', 'оподаль', 'оподаль від', 'оподалік', 'оподалік від', 'опостін', 'опостінь', 'опроче', 'опріч', 'опріче', 'опісля', 'осе', 'оскільки', 'особливо', 'осторонь', 'ось', 'осісьо', 'от', 'ота', 'отак', 'отака', 'отаке', 'отакий', 'отаким', 'отакими', 'отаких', 'отакого', 'отакому', 'отакою', 'отакої', 'отаку', 'отакі', 'отакій', 'отакім', 'отакісінька', 'отакісіньке', 'отакісінький', 'отакісіньким', 'отакісінькими', 'отакісіньких', 'отакісінького', 'отакісінькому', 'отакісінькою', 'отакісінької', 'отакісіньку', 'отакісінькі', 'отакісінькій', 'отакісінькім', 'отам', 'оте', 'отже', 'отим', 'отими', 'отих', 'ото', 'отого', 'отож', 'отой', 'отому', 'отою', 'отої', 'отсе', 'оттак', 'отто', 'оту', 'отут', 'оті', 'отій', 'отім', 'отією', 'отієї', 'ох', 'оце', 'оцей', 'оцим', 'оцими', 'оцих', 'оцього', 'оцьому', 'оцю', 'оця', 'оці', 'оцій', 'оцім', 'оцією', 'оцієї', 'п', "п'я", "п'ятий", "п'ятнадцятий", "п'ятнадцять", "п'ять", 'па', 'пад', 'пак', 'пек', 'перед', 'передо', 'переді', 'перетака', 'перетаке', 'перетакий', 'перетаким', 'перетакими', 'перетаких', 'перетакого', 'перетакому', 'перетакою', 'перетакої', 'перетаку', 'перетакі', 'перетакій', 'перетакім', 'перший', 'пиж', 'плі', 'по', 'поблизу', 'побік', 'побіля', 'побіч', 'поверх', 'повз', 'повздовж', 'повинно', 'повище', 'повсюди', 'повсюдно', 'подаль від', 'подалі від', 'подекуди', 'подеяка', 'подеяке', 'подеякий', 'подеяким', 'подеякими', 'подеяких', 'подеякого', 'подеякому', 'подеякою', 'подеякої', 'подеяку', 'подеякі', 'подеякій', 'подеякім', 'подовж', 'подібно до', 'поз', 'поза', 'позад', 'позаду', 'позата', 'позате', 'позатим', 'позатими', 'позатих', 'позатого', 'позатой', 'позатому', 'позатою', 'позатої', 'позату', 'позаті', 'позатій', 'позатім', 'позатією', 'позатієї', 'позаяк', 'поздовж', 'поки', 'покрай', 'покіль', 'помежи', 'помимо', 'поміж', 'помість', 'понад', 'понадо', 'понаді', 'понижче', 'пообіч', 'поодаль від', 'поодалік від', 'поперед', 'попереду', 'поперек', 'попліч', 'попри', 'попросту', 'попід', 'пора', 'поруч', 'поряд', 'поряд з', 'порівняно з', 'посеред', 'посередині', 'потрібно', 'потім', 'поуз', 'початку', 'почерез', 'праворуч від', 'пред', 'предо', 'преді', 'прекрасно', 'прецінь', 'при', 'притому', 'причому', 'причім', 'про', 'проз', 'промеж', 'проміж', 'просто', 'проте', 'проти', 'против', 'противно', 'протягом', 'пря', 'пріч', 'пхе', 'пху', 'пі', 'пів', 'півперек', 'під', 'під знаком', 'під приводом', 'під час', 'підо', 'пізніше', 'пім', 'пір', 'після', 'р', 'ради', 'раз', 'разом з', 'разу', 'рано', 'раніш', 'раніш від', 'раніше', 'раніше від', 'раптом', 'ре', 'рет', 'риж', 'рим', 'рип', 'роб', 'року', 'років', 'рос', 'рох', 'році', 'рус', 'рух', 'руч', 'рік', 'с', 'саж', 'саз', 'сак', 'сам', 'сама', 'саме', 'сами', 'самий', 'самим', 'самими', 'самих', 'само', 'самого', 'самому', 'самою', 'самої', 'саму', 'самі', 'самій', 'самім', 'сап', 'сас', 'свого', 'свою', 'своя', 'своє', 'своєму', 'своєю', 'своєї', 'свої', 'своїй', 'своїм', 'своїми', 'своїх', 'свій', 'се', 'себе', 'себто', 'сей', 'сен', 'серед', 'середи', 'середу', 'сеч', 'си', 'сив', 'сиг', 'сиз', 'сик', 'сиріч', 'сих', 'сказав', 'сказала', 'сказати', 'скрізь', 'скільки', 'скільки-то', 'скількись', 'скільком', 'скількома', 'скількомась', 'скількомось', 'скількомсь', 'скількох', 'скількохось', 'скількохсь', 'сли', 'слідом за', 'соб', 'собою', 'собі', 'соп', 'спасибі', 'спереду', 'спочатку', 'справ', 'справді', 'став', 'стосовно', 'стільки', 'стільком', 'стількома', 'стількох', 'су', 'судячи з', 'супроти', 'супротив', 'суть', 'суч', 'суш', 'сьогодні', 'сьомий', 'сюди', 'ся', 'сяг', 'сяк', 'сяка', 'сяке', 'сякий', 'сяким', 'сякими', 'сяких', 'сякого', 'сякому', 'сякою', 'сякої', 'сяку', 'сякі', 'сякій', 'сякім', 'сям', 'сі', 'сім', 'сімнадцятий', 'сімнадцять', 'сіп', 'т', 'та', 'таж', 'так', 'така', 'таке', 'такенна', 'такенне', 'такенний', 'такенним', 'такенними', 'такенних', 'такенного', 'такенному', 'такенною', 'такенної', 'такенну', 'такенні', 'такенній', 'такеннім', 'таки', 'такий', 'таким', 'такими', 'таких', 'такого', 'також', 'такому', 'такою', 'такої', 'таку', 'такі', 'такій', 'такім', 'такісінька', 'такісіньке', 'такісінький', 'такісіньким', 'такісінькими', 'такісіньких', 'такісінького', 'такісінькому', 'такісінькою', 'такісінької', 'такісіньку', 'такісінькі', 'такісінькій', 'такісінькім', 'тал', 'там', 'тамки', 'тамта', 'тамте', 'тамтим', 'тамтими', 'тамтих', 'тамтого', 'тамтой', 'тамтому', 'тамтою', 'тамтої', 'тамту', 'тамті', 'тамтій', 'тамтім', 'тамтією', 'тамтієї', 'тар', 'тат', 'таш', 'тва', 'твого', 'твою', 'твоя', 'твоє', 'твоєму', 'твоєю', 'твоєї', 'твої', 'твоїй', 'твоїм', 'твоїми', 'твоїх', 'твій', 'те', 'тебе', 'тег', 'теж', 'тем', 'тепер', 'теперечки', 'тес', 'теф', 'теє', 'ти', 'тик', 'тил', 'тим', 'тими', 'тисяч', 'тих', 'то', 'тобою', 'тобто', 'тобі', 'того', 'тоді', 'тож', 'той', 'тол', 'тому', 'тому що', 'тот', 'тощо', 'тою', 'тої', 'тра', 'тре', 'треба', 'третій', 'три', 'тринадцятий', 'тринадцять', 'трохи', 'тс', 'тсс', 'ту', 'туди', 'тудою', 'туп', 'тут', 'тутеньки', 'тутечки', 'тутки', 'туф', 'туц', 'тю', 'тюг', 'тюп', 'тяг', 'тяж', 'тям', 'тяп', 'ті', 'тій', 'тільки', 'тім', 'тією', 'у', 'у бік', 'у вигляді', 'у випадку', 'у відповідності до', 'у відповідь на', 'у залежності від', "у зв'язку з", 'у міру', 'у напрямі до', 'у порівнянні з', 'у процесі', 'у результаті', 'у ролі', 'у силу', 'у сторону', 'у супроводі', 'у ході', 'ув', 'увесь', 'уві', 'угу', 'уже', 'узбіч', 'уздовж', 'укр', 'ум', 'унаслідок', 'униз', 'унизу', 'унт', 'уперед', 'уподовж', 'упоперек', 'упритиск до', 'упритул до', 'упродовж', 'упрост', 'ус', 'усе', 'усередині', 'услід', 'услід за', 'усупереч', 'усього', 'усьому', 'усю', 'усюди', 'уся', 'усяк', 'усяка', 'усяке', 'усякий', 'усяким', 'усякими', 'усяких', 'усякого', 'усякому', 'усякою', 'усякої', 'усяку', 'усякі', 'усякій', 'усякім', 'усі', 'усій', 'усіляка', 'усіляке', 'усілякий', 'усіляким', 'усілякими', 'усіляких', 'усілякого', 'усілякому', 'усілякою', 'усілякої', 'усіляку', 'усілякі', 'усілякій', 'усілякім', 'усім', 'усіма', 'усіх', 'усією', 'усієї', 'утім', 'ух', 'ф', "ф'ю", 'фа', 'фаг', 'фай', 'фат', 'фе', 'фед', 'фез', 'фес', 'фет', 'фзн', 'фоб', 'фот', 'фра', 'фру', 'фу', 'фук', 'фур', 'фус', 'фіш', 'х', 'ха', 'хаз', 'хай', 'хап', 'хат', 'хащ', 'хе', 'хет', 'хи', 'хиб', 'хм', 'хо', 'хов', 'хол', 'хон', 'хоп', 'хор', 'хотіти', 'хоч', 'хоча', 'хочеш', 'хро', 'хрю', 'хто', 'хтось', 'ху', 'хуз', 'хук', 'хух', 'хху', 'хіба', 'ц', 'це', 'цебто', 'цей', 'цеп', 'ци', 'цим', 'цими', 'цир', 'цих', 'цло', 'цоб', 'цок', 'цоп', 'цор', 'цс', 'цсс', 'цуг', 'цур', 'цуц', 'цього', 'цьому', 'цю', 'цюк', 'ця', 'цяв', 'цяп', 'ці', 'цід', 'цій', 'цім', 'ціною', 'цією', 'цієї', 'ч', 'чал', 'чар', 'час', 'часто', 'частіше', 'часу', 'чах', 'чей', 'чень', 'через', 'четвертий', 'чи', 'чий', 'чийого', 'чийогось', 'чийому', 'чийомусь', 'чийсь', 'чик', 'чим', 'чимось', 'чимсь', 'чир', 'численна', 'численне', 'численний', 'численним', 'численними', 'численних', 'численні', 'чию', 'чиюсь', 'чия', 'чиясь', 'чиє', 'чиєму', 'чиємусь', 'чиєсь', 'чиєю', 'чиєюсь', 'чиєї', 'чиєїсь', 'чиї', 'чиїй', 'чиїйсь', 'чиїм', 'чиїми', 'чиїмись', 'чиїмось', 'чиїмсь', 'чиїсь', 'чиїх', 'чиїхось', 'чиїхсь', 'чля', 'чого', 'чогось', 'чом', 'чому', 'чомусь', 'чон', 'чоп', 'чортзна', 'чос', 'чотири', 'чотирнадцятий', 'чотирнадцять', 'чу', 'чум', 'чур', 'чш', 'чім', 'чімось', 'чімсь', 'чіт', 'ш', 'ша', 'шаг', 'шал', 'шам', 'шво', 'шед', 'шен', 'шиз', 'шир', 'шляхом', 'шостий', 'шістнадцятий', 'шістнадцять', 'шість', 'щ', 'ще', 'щем', 'щеп', 'щип', 'щир', 'що', 'щоб', 'щоби', 'щодо', 'щойно', 'щоправда', 'щось', 'щі', 'ь', 'ю', 'юз', 'юн', 'юнь', 'юс', 'ют', 'юхт', 'я', 'яв', 'яд', 'яз', 'язь', 'як', 'яка', 'якась', 'якби', 'яке', 'якесь', 'який', 'якийсь', 'яким', 'якими', 'якимись', 'якимось', 'якимсь', 'яких', 'якихось', 'якихсь', 'якого', 'якогось', 'якому', 'якомусь', 'якось', 'якою', 'якоюсь', 'якої', 'якоїсь', 'якраз', 'яку', 'якусь', 'якщо', 'які', 'якій', 'якійсь', 'якім', 'якімсь', 'якісь', 'ял', 'ям', 'ян', 'янь', 'яо', 'яп', 'ярл', 'ясь', 'ять', 'є', 'єр', 'єси', 'і', 'ібн', 'ід', 'із', 'із-за', 'із-під', 'іззаду', 'ізм', 'ізсередини', 'ік', 'ікс', 'ікт', "ім'я", 'імовірно', 'інакша', 'інакше', 'інакший', 'інакшим', 'інакшими', 'інакших', 'інакшого', 'інакшому', 'інакшою', 'інакшої', 'інакшу', 'інакші', 'інакшій', 'інакшім', 'інколи', 'іноді', 'інша', 'інше', 'інший', 'іншим', 'іншими', 'інших', 'іншого', 'іншому', 'іншою', 'іншої', 'іншу', 'інші', 'іншій', 'іншім', 'інь', 'іч', 'іще', 'ї', 'їдь', 'їй', 'їм', 'їх', 'їхнього', 'їхньому', 'їхньою', 'їхньої', 'їхню', 'їхня', 'їхнє', 'їхні', 'їхній', 'їхнім', 'їхніми', 'їхніх', 'її', 'ґ'}
        # fmt: on
    elif custom_stopwords:
        stop_words = custom_stopwords
    else:
        stop_words = stopwords.words(language)

    all_words = ""
    for message_ in json_gen:
        if isinstance(message_.get("text", False), str):
            text = re.sub(r"\W+", " ", message_["text"])  # deleting symbols
            text = text.replace("\n", "")  # deleting indents
            print(f"({message_['date']}) {text.lower().strip()}", end="\r")
            words = nltk.word_tokenize(text.lower())
            without_stop_words = [word for word in words if not word in stop_words]
            for word in without_stop_words:
                all_words += f"{word} "
    wordcloud: WordCloud = WordCloud(
        width=2000,
        height=1500,
        random_state=1,
        background_color="black",
        margin=20,
        colormap="Pastel1",
        collocations=False,
        max_words=max_words,
        stopwords=stop_words,
    ).generate(all_words)
    print(wordcloud.words_)
    if plot:
        plot_wordcloud(wordcloud)
    return wordcloud



from typing import AsyncGenerator

from pyrogram import Client as PyroClient
import datetime
from collections import Counter
from numpy import inf
from pyrogram.types import User, Message


class Client(PyroClient):
    def __init__(
        self,
        name: str,
        api_id: int | str | None = None,
        api_hash: str = None,
        ipv6: bool = False,
        proxy: dict = None,
        test_mode: bool = False,
        bot_token: str = None,
        session_string: str = None,
        in_memory: bool = None,
        phone_number: str = None,
        phone_code: str = None,
        password: str = None,
        plugins: dict = None,
        no_updates: bool = None,
        takeout: bool = None,
        hide_password: bool = True,
    ):
        super().__init__(
            name=name,
            api_id=api_id,
            api_hash=api_hash,
            ipv6=ipv6,
            proxy=proxy,
            test_mode=test_mode,
            bot_token=bot_token,
            session_string=session_string,
            in_memory=in_memory,
            phone_number=phone_number,
            phone_code=phone_code,
            password=password,
            plugins=plugins,
            no_updates=no_updates,
            takeout=takeout,
            hide_password=hide_password,
        )

    async def chat_id_by_name(self, chat_name) -> int:
        async for dialog in self.get_dialogs():
            if (
                dialog.chat.title or f"{dialog.chat.first_name} {dialog.chat.last_name}"
            ) == chat_name:
                return dialog.chat.id

    async def chat_history_agen(
        self, chat_id, n=inf, datetime_until=datetime.datetime(1990, 1, 1)
    ) -> AsyncGenerator:
        """Creates an async generator of messages, that may contain a reactions"""
        async for message in self.get_chat_history(chat_id=chat_id, limit=n):
            yield message
            now = message.date.date()
            if datetime.datetime(now.year, now.month, now.day) < datetime_until:
                break

    async def get_message_reactions(self, chat_name, n=inf) -> Message:
        chat_id: int = await self.chat_id_by_name(chat_name)
        counter = 0
        async for message in self.chat_history_agen(chat_id, n=n):
            if all(
                [
                    message.reactions,
                    message.forward_date is None,
                    message.media is None,
                    message.from_user,
                ]
            ):
                yield message
                counter += 1

    async def count_user_text_messages(
        self, chat_name: str, username_list: list[User], n=inf
    ) -> Counter:
        chat_id: int = await self.chat_id_by_name(chat_name)
        message_counter = Counter()
        async for message in self.chat_history_agen(chat_id, n=n):
            if all(
                [
                    message.forward_date is None,
                    message.media is None,
                    message.from_user,
                    message.from_user.username in username_list,
                ]
            ):
                position: int = username_list.index(message.from_user.username)
                message_counter.update([username_list[position]])
            print(f"Message Counter: {message_counter}", end="\r")
        return message_counter



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['telenal']

package_data = \
{'': ['*']}

install_requires = \
['Pyrogram>=2.0.59,<3.0.0',
 'TgCrypto>=1.2.4,<2.0.0',
 'matplotlib>=3.6.1,<4.0.0',
 'nltk>=3.7,<4.0',
 'numpy>=1.23.4,<2.0.0',
 'pandas>=1.5.1,<2.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'scipy>=1.9.3,<2.0.0',
 'stop-words>=2018.7.23,<2019.0.0',
 'wordcloud>=1.8.2,<2.0.0']

setup_kwargs = {
    'name': 'telegram-anal',
    'version': '0.1.1',
    'description': 'A fun tool to play with Telegram chats',
    'long_description': '# Telegram analysis ✨ \n\nTelegram analysis is a python package that helps you have fun and make sense of different chat data you have. \nBased on pyrogram, its mission is to help you have fun with Telegram and it\'s functions.\nWhether it is about playing with your group chat dialogs or analysing the group dynamic of your business, personal chats or even your saved messages.\n\nBe creative and build on top of ready solutions! 🧠\n\nCurrent functionality (version 0.1.1):\n\n✨🌠 `wordcloud_json()`Create fun wordclouds out of chats \n\n💬🗣 `who_breaks_silence_json()` Analyze who tends to break the silence in the chats and write first\n\n🤍❤ `measure_top_reactions()` measure who\'s messages are most reacted to and choose specific reactions!\n\n\n___\n### Installation:\n```\npip install telegram-anal\n```\n\n### 🛠 Setup:\n\nThis library uses Pyrogram for handling Telegram, so you will need to authenticate standardly by:\n1. Create an app and copy your api_id and api_hash from https://my.telegram.org/\n\n### Example:\n```python\nimport asyncio\nfrom telenal.client import Client\nfrom telenal.reactions import measure_top_reactions\nfrom telenal.teleplotter import plot_bars_from_dict\n\n\nasync def main():\n    client = Client("my_account", "your_api_id", "your_api_hash")\n    tops = await measure_top_reactions(\n        client,\n        chat_name="Family chat :)",\n        search_emojis=["😁", "💪", "👍"],\n    )\n    plot_bars_from_dict(tops)  # <-- get a bar chart out of the box\n\n\nif __name__ == "__main__":\n    asyncio.run(main())\n\n```\n\nThe library is newly born and needs creative and energetic support. Feel free to contribute!\n\nIdeas for future functionality:\n\n1. Analyze your activity based on sent chatting to identify your chatting patterns\n2. Save messages deleted by someone that texted you and save the secret they wanted to hide\n3. Measure user profiles to identify persons\' most popular tones (NLP may be needed)\n4. Predict when a user is prone to leave a community based on real cases (ML)\n\nRemember to have fun! :)\n\nThis software is made purely for fun. Any illegal or amoral misuse of the software does not make the authors responsible for the consequences. \n\nYours,\nVlad Bilyk\n',
    'author': 'Vladyslav Bilyk',
    'author_email': 'vladozeransky2k@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Salz0/Telegram-anal',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import datetime
from collections import Counter

from dotenv import load_dotenv
from pyrogram.types import Reaction, Message

from .client import Client

load_dotenv()


def weigh_counter_values(counter1: Counter, counter2: Counter) -> dict:
    assert len(counter1) == len(counter2), Exception(
        f"Non-identical datasets ({len(counter1)} != {len(counter2)}"
    )
    assert set(counter1.keys()) == set(counter2.keys()), Exception(
        "Non-identical datasets"
    )
    fin = dict()
    for name in list(counter1.keys()):
        fin[name] = float(f"{counter1[name] / counter2[name] * 100:5f}")
    return fin


def register_text_message_reaction(
    message: Message, reaction_counter: Counter, search_emojis: list
) -> Counter:
    _ = message.date.date()
    if datetime.datetime(_.year, _.month, _.day) < datetime.datetime(2021, 12, 30):
        return reaction_counter
    if not all(
        [
            message.reactions,
            message.forward_date is None,
            message.media is None,
            message.from_user,
        ]
    ):
        return reaction_counter
    reactions_obj, date = message.reactions, message.date.date()
    target_reactions: list[Reaction] = [
        x for x in reactions_obj.reactions if x.emoji in search_emojis
    ]
    # logger.info(f"{message.text or message.reply=}")
    if target_reactions is []:
        return reaction_counter
    print(
        f"Scrolling messages [date, message]: {message.date.date()} {message.text or message.reply}",
        end="\r",
    )
    for target_reaction in target_reactions:
        # logger.info(f"{target_reaction=}")
        for ii in range(target_reaction.count):
            reaction_counter.update([message.from_user.username])
    return reaction_counter


def add_msg_to_user_counter(message: Message, message_counter: Counter):
    if all([message.forward_date is None, message.media is None, message.from_user]):
        message_counter.update([message.from_user.username])
    return message_counter


async def measure_top_reactions(
    client: Client, chat_name: str, search_emojis=None
) -> dict:  # "Чат всіх випускників УАЛ"
    if search_emojis is None:
        search_emojis = ["😁", "🤣"]
    reaction_counter, message_counter = Counter(), Counter()
    async with client:
        chat_id = await client.chat_id_by_name(chat_name)
        async for message in client.chat_history_agen(
            chat_id=chat_id, datetime_until=datetime.datetime(2021, 12, 30)
        ):
            reaction_counter: Counter = register_text_message_reaction(
                message, reaction_counter, search_emojis
            )
            print(f"Reaction Counter: {reaction_counter}", end="\r")
            message_counter = add_msg_to_user_counter(message, message_counter)

        print(f"Message Counter: {message_counter}")
        react_list = list(reaction_counter.keys())
        for x in list(message_counter.keys()):
            if x not in react_list:  # Don't include people without a tag
                del message_counter[x]
            if x is None or message_counter[x] < 40:
                del message_counter[x]
                del reaction_counter[x]
        wcv = weigh_counter_values(reaction_counter, message_counter)
        print(f"Weighed Counter values: {wcv}")
        return wcv
        # plot_bars_from_dict(weighed_counter)
        # plot_normal_distribution(list(weighed_counter.values()))
